<?php

namespace HCM\BTL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;

class TemplatesController extends Crud {

    protected $dataSourceName = 'dadosTemplate';
    protected $dataSourceName2 = 'getTemplate';
    protected $dataSourceName3 = 'buscarIdAtivo';
    protected $dataSourceName4 = 'des_fasetemplate';

    protected $parameterBagColumns = array('NRORG', 'NRTEMPLATE', 'P_NRORG');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters   = $hcmParameters;
        parent::__construct($dataSourceManager);
    }


    public function buscarDados (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->addCondition('P_NRORG','=',$this->hcmParameters->getNrorgTrabalho());
        $filterData->setDataSourceName($this->dataSourceName2);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function buscarIdAtivo (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName3);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $this->parameterBag->set('NRORG', $this->hcmParameters->getNrorgTrabalho());  
    

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

    public function salvarDados (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            foreach($request->getDataSet()->getRows() as &$row) {
                if($row['__is_new'] === true){
                    $row['NRORGINCLUSAO'] = $row['NRORG'];
                    $row['DTINCLUSAO'] = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    $row['NRTEMPLATE'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'DES_TEMPLATE', 1, 12, true);
                }

                $row['DTULTATU'] = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);
                $row['NRORGULTATU'] = $row['NRORG'];
                $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];

                if(!array_key_exists('NRTIPOPROCESSO', $row))
                    $row['NRTIPOPROCESSO'] = 1;
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $aux = $this->dataSourceName;
            $this->dataSourceName = $this->dataSourceName4;
            foreach($request->getDataSet()->getRows() as $row) {
                if(!empty($row['FASES'])){
                    foreach ($row['FASES'] as $fase) {
                        $requestDelete = new DTO\Request\DataSet ((new \Zeedhi\Framework\DataSource\DataSet('des_fasetemplate', array($fase))), $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                        parent::delete($requestDelete, $response);
                    }
                }
            }
            $this->dataSourceName = $aux;
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}